@@ -1,10 +1,14 @@
 # coding=utf-8
 import time
 
+from typing import Any
+from logging import DEBUG, ERROR, INFO
+
 import jc
 
 from simplyblock_core import utils, distr_controller, storage_node_ops
 from simplyblock_core.db_controller import DBController
+from simplyblock_core.fw_api_client import FirewallClient
 from simplyblock_core.models.cluster import Cluster
 from simplyblock_core.models.nvme_device import NVMeDevice, JMDevice
 from simplyblock_core.models.storage_node import StorageNode
@@ -14,6 +18,50 @@ from simplyblock_core.snode_client import SNodeClient
 logger = utils.get_logger(__name__)
 
 
+def check_bdev(name, *, rpc_client=None, bdev_names=None):
+    present = (
+            ((bdev_names is not None) and (name in bdev_names)) or
+            (rpc_client is not None and (rpc_client.get_bdevs(name) is not None))
+    )
+    logger.log(INFO if present else ERROR, f"Checking bdev: {name} ... " + ('ok' if present else 'failed'))
+    return present
+
+
+def check_subsystem(nqn, *, rpc_client=None, nqns=None, ns_uuid=None):
+    if rpc_client:
+        subsystem = subsystems[0] if (subsystems := rpc_client.subsystem_list(nqn)) is not None else None
+    elif nqns:
+        subsystem = nqns.get(nqn)
+    else:
+        raise ValueError('Either rpc_client or nqns must be passed')
+
+    if not subsystem:
+        logger.error(f"Checking subsystem {nqn} ... not found")
+        return False
+
+    logger.info(f"Checking subsystem {nqn} ... ok")
+
+    if ns_uuid:
+        for ns in subsystem['namespaces']:
+            if ns['uuid'] == ns_uuid:
+                namespaces = 1
+                break
+        else:
+            namespaces = 0
+    else:
+        namespaces = len(subsystem['namespaces'])
+    logger.log(INFO if namespaces else ERROR, f"Checking namespaces: {namespaces} ... " + ('ok' if namespaces else 'not found'))
+
+    listeners = subsystem['listen_addresses']
+    if not listeners:
+        logger.error(f"Checking listener for {nqn} ... not found")
+    else:
+        for listener in listeners:
+            logger.info(f"Checking listener {listener['traddr']}:{listener['trsvcid']} ... ok")
+
+    return listeners and namespaces
+
+
 def check_cluster(cluster_id):
     db_controller = DBController()
     st = db_controller.get_storage_nodes_by_cluster_id(cluster_id)
@@ -53,7 +101,7 @@ def check_cluster(cluster_id):
     return result
 
 
-def _check_node_rpc(rpc_ip, rpc_port, rpc_username, rpc_password, timeout=3, retry=2):
+def _check_node_rpc(rpc_ip, rpc_port, rpc_username, rpc_password, timeout=5, retry=2):
     try:
         rpc_client = RPCClient(
             rpc_ip, rpc_port, rpc_username, rpc_password,
@@ -69,7 +117,7 @@ def _check_node_rpc(rpc_ip, rpc_port, rpc_username, rpc_password, timeout=3, ret
 
 def _check_node_api(ip):
     try:
-        snode_api = SNodeClient(f"{ip}:5000", timeout=3, retry=2)
+        snode_api = SNodeClient(f"{ip}:5000", timeout=5, retry=2)
         logger.debug(f"Node API={ip}:5000")
         info, _ = snode_api.info()
         if info:
@@ -82,7 +130,7 @@ def _check_node_api(ip):
 
 def _check_spdk_process_up(ip, rpc_port):
     try:
-        snode_api = SNodeClient(f"{ip}:5000", timeout=3, retry=2)
+        snode_api = SNodeClient(f"{ip}:5000", timeout=5, retry=2)
         logger.debug(f"Node API={ip}:5000")
         is_up, _ = snode_api.spdk_process_is_up(rpc_port)
         logger.debug(f"SPDK is {is_up}")
@@ -94,15 +142,15 @@ def _check_spdk_process_up(ip, rpc_port):
 
 def _check_port_on_node(snode, port_id):
     try:
-        snode_api = SNodeClient(f"{snode.mgmt_ip}:5000", timeout=3, retry=2)
-        iptables_command_output, _ = snode_api.get_firewall(snode.rpc_port)
+        fw_api = FirewallClient(f"{snode.mgmt_ip}:5001", timeout=5, retry=2)
+        iptables_command_output, _ = fw_api.get_firewall(snode.rpc_port)
         result = jc.parse('iptables', iptables_command_output)
         for chain in result:
             if chain['chain'] in ["INPUT", "OUTPUT"]:
                 for rule in chain['rules']:
                     if str(port_id) in rule['options']:
                         action = rule['target']
-                        if action == "DROP":
+                        if action in ["DROP", "REJECT"]:
                             return False
 
         return True
@@ -118,6 +166,165 @@ def _check_node_ping(ip):
     else:
         return False
 
+def _check_node_hublvol(node: StorageNode, node_bdev_names=None, node_lvols_nqns=None):
+    if not node.hublvol:
+        logger.error(f"Node {node.get_id()} does not have a hublvol")
+        return False
+
+    logger.info(f"Checking Hublvol: {node.hublvol.bdev_name} on node {node.get_id()}")
+    db_controller = DBController()
+
+    passed = True
+    try:
+        rpc_client = RPCClient(
+            node.mgmt_ip, node.rpc_port, node.rpc_username, node.rpc_password, timeout=5, retry=1)
+
+        if not node_bdev_names:
+            node_bdev_names = {}
+            ret = rpc_client.get_bdevs()
+            if ret:
+                for b in ret:
+                    node_bdev_names[b['name']] = b
+                    for al in b['aliases']:
+                        node_bdev_names[al] = b
+
+        if not node_lvols_nqns:
+            node_lvols_nqns = {}
+            ret = rpc_client.subsystem_list()
+            for sub in ret:
+                node_lvols_nqns[sub['nqn']] = sub
+
+        passed &= check_bdev(node.hublvol.bdev_name, bdev_names=node_bdev_names)
+        passed &= check_subsystem(node.hublvol.nqn, nqns=node_lvols_nqns)
+
+        try:
+            cl = db_controller.get_cluster_by_id(node.cluster_id)
+        except KeyError:
+            logger.error(f"Cluster with id {node.cluster_id} not found")
+            return False
+
+        ret = rpc_client.bdev_lvol_get_lvstores(node.lvstore)
+        if ret:
+            logger.info(f"Checking lvstore: {node.lvstore} ... ok")
+            lvs_info = ret[0]
+            logger.info("lVol store Info:")
+            lvs_info_dict = []
+            expected: dict[str, Any] = {}
+            expected["lvs leadership"] = True
+            expected["lvs_primary"] = True
+            expected["lvs_read_only"] = False
+            expected["name"] = node.lvstore
+            expected["base_bdev"] = node.raid
+            expected["block_size"] = cl.blk_size
+            expected["cluster_size"] = cl.page_size_in_blocks
+
+            for k, v in lvs_info.items():
+                if k in expected:
+                    value = expected[k] == v
+                    lvs_info_dict.append({"Key": k, "Value": v, "expected": value})
+                    if value is bool and v is False:
+                        passed = False
+                else:
+                    lvs_info_dict.append({"Key": k, "Value": v, "expected": " "})
+            for line in utils.print_table(lvs_info_dict).splitlines():
+                logger.info(line)
+
+    except Exception as e:
+        logger.exception(e)
+    return passed
+
+
+def _check_sec_node_hublvol(node: StorageNode, node_bdev=None, node_lvols_nqns=None, auto_fix=False):
+    db_controller = DBController()
+    try:
+        primary_node = db_controller.get_storage_node_by_id(node.lvstore_stack_secondary_1)
+    except KeyError:
+        logger.exception("Primary node not found")
+        return False
+    
+    if not primary_node.hublvol:
+        logger.error(f"Primary node {primary_node.get_id()} does not have a hublvol")
+        return False
+    
+    logger.info(f"Checking secondary Hublvol: {primary_node.hublvol.bdev_name} on node {node.get_id()}")
+
+    passed = True
+    try:
+        rpc_client = RPCClient(
+            node.mgmt_ip, node.rpc_port, node.rpc_username, node.rpc_password, timeout=5, retry=1)
+
+        if not node_bdev:
+            node_bdev = {}
+            ret = rpc_client.get_bdevs()
+            if ret:
+                for b in ret:
+                    node_bdev[b['name']] = b
+                    for al in b['aliases']:
+                        node_bdev[al]= b
+            else:
+                node_bdev = []
+
+        if not node_lvols_nqns:
+            node_lvols_nqns = {}
+            ret = rpc_client.subsystem_list()
+            for sub in ret:
+                node_lvols_nqns[sub['nqn']] = sub
+
+
+        ret = rpc_client.bdev_nvme_controller_list(primary_node.hublvol.bdev_name)
+        passed = bool(ret)
+        logger.info(f"Checking controller: {primary_node.hublvol.bdev_name} ... {passed}")
+
+        if not passed and auto_fix and primary_node.lvstore_status == "ready" \
+                and primary_node.status in [StorageNode.STATUS_ONLINE, StorageNode.STATUS_DOWN]:
+            try:
+                node.connect_to_hublvol(primary_node)
+            except Exception as e:
+                logger.error("Error establishing hublvol: %s", e)
+            ret = rpc_client.bdev_nvme_controller_list(primary_node.hublvol.bdev_name)
+            passed = bool(ret)
+            logger.info(f"Checking controller: {primary_node.hublvol.bdev_name} ... {passed}")
+
+        passed &= check_bdev(primary_node.hublvol.get_remote_bdev_name(), bdev_names=node_bdev)
+        try:
+            cl = db_controller.get_cluster_by_id(node.cluster_id)
+        except KeyError:
+            logger.error(f"Cluster with id {node.cluster_id} not found")
+            return False
+        ret = rpc_client.bdev_lvol_get_lvstores(primary_node.lvstore)
+        if ret:
+            logger.info(f"Checking lvstore: {primary_node.lvstore} ... ok")
+            lvs_info = ret[0]
+            logger.info("lVol store Info:")
+            lvs_info_dict = []
+            expected: dict [str, Any] = {}
+            expected["name"] = primary_node.lvstore
+            expected["lvs leadership"] = False
+            expected["lvs_secondary"] = True
+            expected["lvs_read_only"] = False
+            expected["lvs_redirect"] = True
+            expected["remote_bdev"] = primary_node.hublvol.get_remote_bdev_name()
+            expected["connect_state"] = True
+            expected["base_bdev"] = primary_node.raid
+            expected["block_size"] = cl.blk_size
+            expected["cluster_size"] = cl.page_size_in_blocks
+
+            for k, v in lvs_info.items():
+
+                if k in expected:
+                    value = expected[k] == v
+                    lvs_info_dict.append({"Key": k, "Value": v, "expected": value})
+                    if value is bool and value is False:
+                        passed = False
+
+                else:
+                    lvs_info_dict.append({"Key": k, "Value": v, "expected": " "})
+            for line in utils.print_table(lvs_info_dict).splitlines():
+                logger.info(line)
+    except Exception as e:
+        logger.exception(e)
+    return passed
+
 
 def _check_node_lvstore(
         lvstore_stack, node, auto_fix=False, node_bdev_names=None, stack_src_node=None):
@@ -125,7 +332,7 @@ def _check_node_lvstore(
     lvstore_check = True
     logger.info(f"Checking distr stack on node : {node.get_id()}")
     rpc_client = RPCClient(
-        node.mgmt_ip, node.rpc_port, node.rpc_username, node.rpc_password, timeout=3, retry=1)
+        node.mgmt_ip, node.rpc_port, node.rpc_username, node.rpc_password, timeout=5, retry=1)
     cluster = db_controller.get_cluster_by_id(node.cluster_id)
     if cluster.status not in [Cluster.STATUS_ACTIVE, Cluster.STATUS_DEGRADED, Cluster.STATUS_READONLY]:
         auto_fix = False
@@ -157,7 +364,7 @@ def _check_node_lvstore(
     for distr in distribs_list:
         if distr in node_bdev_names:
             logger.info(f"Checking distr bdev : {distr} ... ok")
-            logger.info(f"Checking distr JM names:")
+            logger.info("Checking distr JM names:")
             if distr in node_distribs_list:
                 jm_names = storage_node_ops.get_node_jm_names(node)
             elif stack_src_node:
@@ -180,7 +387,7 @@ def _check_node_lvstore(
                         for result in results:
                             if result['Results'] == 'failed':
                                 if result['Kind'] == "Device":
-                                    if result['Reported Status']:
+                                    if result['Found Status']:
                                         dev = db_controller.get_storage_device_by_id(result['UUID'])
                                         if dev.status == NVMeDevice.STATUS_ONLINE:
                                             name = f"remote_{dev.alceml_bdev}"
@@ -226,9 +433,10 @@ def _check_node_lvstore(
 
 def check_node(node_id, with_devices=True):
     db_controller = DBController()
-    snode = db_controller.get_storage_node_by_id(node_id)
-    if not snode:
-        logger.error("node not found")
+    try:
+        snode = db_controller.get_storage_node_by_id(node_id)
+    except KeyError:
+        logger.exception("node not found")
         return False
 
     if snode.status in [StorageNode.STATUS_OFFLINE, StorageNode.STATUS_REMOVED]:
@@ -262,10 +470,12 @@ def check_node(node_id, with_devices=True):
             data_nics_check &= ping_check
 
     if snode.lvstore_stack_secondary_1:
-        n = db_controller.get_storage_node_by_id(snode.lvstore_stack_secondary_1)
-        if n:
+        try:
+            n = db_controller.get_storage_node_by_id(snode.lvstore_stack_secondary_1)
             lvol_port_check = _check_port_on_node(snode, n.lvol_subsys_port)
             logger.info(f"Check: node {snode.mgmt_ip}, port: {n.lvol_subsys_port} ... {lvol_port_check}")
+        except KeyError:
+            pass
     if not snode.is_secondary_node:
         lvol_port_check = _check_port_on_node(snode, snode.lvol_subsys_port)
         logger.info(f"Check: node {snode.mgmt_ip}, port: {snode.lvol_subsys_port} ... {lvol_port_check}")
@@ -283,6 +493,7 @@ def check_node(node_id, with_devices=True):
         logger.info("Skipping devices checks because RPC check failed")
     else:
         logger.info(f"Node device count: {len(snode.nvme_devices)}")
+        print("*" * 100)
         for dev in snode.nvme_devices:
             if dev.status in [NVMeDevice.STATUS_ONLINE, NVMeDevice.STATUS_UNAVAILABLE]:
                 ret = check_device(dev.get_id())
@@ -292,20 +503,16 @@ def check_node(node_id, with_devices=True):
             print("*" * 100)
 
         logger.info(f"Node remote device: {len(snode.remote_devices)}")
+        print("*" * 100)
         rpc_client = RPCClient(
             snode.mgmt_ip, snode.rpc_port,
             snode.rpc_username, snode.rpc_password,
-            timeout=3, retry=1)
+            timeout=5, retry=1)
         for remote_device in snode.remote_devices:
-            ret = rpc_client.get_bdevs(remote_device.remote_bdev)
-            if ret:
-                logger.info(f"Checking bdev: {remote_device.remote_bdev} ... ok")
-            else:
-                logger.info(f"Checking bdev: {remote_device.remote_bdev} ... not found")
-            node_remote_devices_check &= bool(ret)
+            node_remote_devices_check &= check_remote_device(remote_device.get_id(), snode)
+            print("*" * 100)
 
         if snode.jm_device:
-            print("*" * 100)
             jm_device = snode.jm_device
             logger.info(f"Node JM: {jm_device.get_id()}")
             ret = check_jm_device(jm_device.get_id())
@@ -319,36 +526,51 @@ def check_node(node_id, with_devices=True):
             print("*" * 100)
             logger.info(f"Node remote JMs: {len(snode.remote_jm_devices)}")
             for remote_device in snode.remote_jm_devices:
-                ret = rpc_client.get_bdevs(remote_device.remote_bdev)
-                if ret:
-                    logger.info(f"Checking bdev: {remote_device.remote_bdev} ... ok")
-                else:
-                    logger.info(f"Checking bdev: {remote_device.remote_bdev} ... not found")
-                node_remote_devices_check &= bool(ret)
+
+                name = f'remote_{remote_device.jm_bdev}n1'
+                bdev_info = rpc_client.get_bdevs(name)
+                logger.log(INFO if bdev_info else ERROR,
+                           f"Checking bdev: {name} ... " + ('ok' if bdev_info else 'failed'))
+                node_remote_devices_check &= bool(bdev_info)
+
+                controller_info = rpc_client.bdev_nvme_controller_list(f'remote_{remote_device.jm_bdev}')
+                if controller_info:
+                    addr = controller_info[0]['ctrlrs'][0]['trid']['traddr']
+                    port = controller_info[0]['ctrlrs'][0]['trid']['trsvcid']
+                    logger.info(f"IP Address: {addr}:{port}")
+
+                if remote_device.nvmf_multipath:
+                    if controller_info and "alternate_trids" in controller_info[0]['ctrlrs'][0]:
+                        addr = controller_info[0]['ctrlrs'][0]['alternate_trids'][0]['traddr']
+                        port = controller_info[0]['ctrlrs'][0]['alternate_trids'][0]['trsvcid']
+                        logger.info(f"IP Address: {addr}:{port}")
+
+                    if bdev_info:
+                        logger.info(f"multipath policy: {bdev_info[0]['driver_specific']['mp_policy']}")
 
         print("*" * 100)
         if snode.lvstore_stack:
             lvstore_stack = snode.lvstore_stack
             lvstore_check &= _check_node_lvstore(lvstore_stack, snode)
-
+            print("*" * 100)
             if snode.secondary_node_id:
                 second_node_1 = db_controller.get_storage_node_by_id(snode.secondary_node_id)
                 if second_node_1.status == StorageNode.STATUS_ONLINE:
                     lvstore_check &= _check_node_lvstore(lvstore_stack, second_node_1, stack_src_node=snode)
-
-        # if snode.lvstore_stack_secondary_1:
-        #     for node in db_controller.get_storage_nodes():
-        #         if node.secondary_node_id == snode.get_id() and node.status == StorageNode.STATUS_ONLINE:
-        #             logger.info(f"Checking stack from node : {node.get_id()}")
-        #             lvstore_check &= _check_node_lvstore(node.lvstore_stack, snode)
+                    print("*" * 100)
+                lvstore_check &= _check_node_hublvol(snode)
+                if second_node_1.status == StorageNode.STATUS_ONLINE:
+                    print("*" * 100)
+                    lvstore_check &= _check_sec_node_hublvol(second_node_1)
 
     return is_node_online and node_devices_check and node_remote_devices_check and lvstore_check
 
 
 def check_device(device_id):
     db_controller = DBController()
-    device = db_controller.get_storage_device_by_id(device_id)
-    if not device:
+    try:
+        device = db_controller.get_storage_device_by_id(device_id)
+    except KeyError:
         # is jm device ?
         for node in db_controller.get_storage_nodes():
             if node.jm_device and node.jm_device.get_id() == device_id:
@@ -357,9 +579,10 @@ def check_device(device_id):
         logger.error("device not found")
         return False
 
-    snode = db_controller.get_storage_node_by_id(device.node_id)
-    if not snode:
-        logger.error("node not found")
+    try:
+        snode = db_controller.get_storage_node_by_id(device.node_id)
+    except KeyError:
+        logger.exception("node not found")
         return False
 
     if snode.status in [StorageNode.STATUS_OFFLINE, StorageNode.STATUS_REMOVED]:
@@ -388,28 +611,20 @@ def check_device(device_id):
         for bdev in bdevs_stack:
             if not bdev:
                 continue
-            ret = rpc_client.get_bdevs(bdev)
-            if ret:
-                logger.debug(f"Checking bdev: {bdev} ... ok")
-            else:
-                logger.error(f"Checking bdev: {bdev} ... not found")
+
+            if not check_bdev(bdev, rpc_client=rpc_client):
                 problems += 1
                 passed = False
-                # return False
+
         logger.info(f"Checking Device's BDevs ... ({(len(bdevs_stack)-problems)}/{len(bdevs_stack)})")
 
-        ret = rpc_client.subsystem_list(device.nvmf_nqn)
-        logger.debug(f"Checking subsystem: {device.nvmf_nqn}")
-        if ret:
-            logger.info(f"Checking subsystem ... ok")
-        else:
-            logger.info(f"Checking subsystem: ... not found")
-            passed = False
+        passed &= check_subsystem(device.nvmf_nqn, rpc_client=rpc_client)
 
-        if device.status == NVMeDevice.STATUS_ONLINE:
-            logger.info("Checking other node's connection to this device...")
-            ret = check_remote_device(device_id)
-            # passed &= ret
+        # if device.status == NVMeDevice.STATUS_ONLINE:
+        #     logger.info("Checking other node's connection to this device...")
+        #     ret = check_remote_device(device_id)
+        # if not ret:
+        #         logger.warning(f"Remote device {device_id} is not accessible from other nodes")#     # passed &= ret
 
     except Exception as e:
         logger.error(f"Failed to connect to node's SPDK: {e}")
@@ -418,31 +633,49 @@ def check_device(device_id):
     return passed
 
 
-def check_remote_device(device_id):
+def check_remote_device(device_id, target_node=None):
     db_controller = DBController()
-    device = db_controller.get_storage_device_by_id(device_id)
-    if not device:
+    try:
+        device = db_controller.get_storage_device_by_id(device_id)
+    except KeyError:
         logger.error("device not found")
         return False
-    snode = db_controller.get_storage_node_by_id(device.node_id)
-    if not snode:
-        logger.error("node not found")
+
+    try:
+        snode = db_controller.get_storage_node_by_id(device.node_id)
+    except KeyError:
+        logger.exception("node not found")
         return False
 
     result = True
-    for node in db_controller.get_storage_nodes_by_cluster_id(snode.cluster_id):
+    if target_node:
+        nodes = [target_node]
+    else:
+        nodes = db_controller.get_storage_nodes_by_cluster_id(snode.cluster_id)
+    for node in nodes:
         if node.status == StorageNode.STATUS_ONLINE:
             if node.get_id() == snode.get_id():
                 continue
-            logger.info(f"Connecting to node: {node.get_id()}")
+            logger.info(f"Checking device: {device_id}")
             rpc_client = RPCClient(node.mgmt_ip, node.rpc_port, node.rpc_username, node.rpc_password, timeout=5, retry=1)
-            name = f"remote_{device.alceml_bdev}n1"
-            ret = rpc_client.get_bdevs(name)
-            if ret:
-                logger.info(f"Checking bdev: {device.alceml_bdev} ... ok")
-            else:
-                logger.info(f"Checking bdev: {device.alceml_bdev} ... not found")
-                result = False
+            name = f'remote_{device.alceml_bdev}n1'
+            bdev_info = rpc_client.get_bdevs(name)
+            logger.log(DEBUG if bdev_info else ERROR, f"Checking bdev: {name} ... " + ('ok' if bdev_info else 'failed'))
+            result &= bool(bdev_info)
+            controller_info = rpc_client.bdev_nvme_controller_list(f'remote_{device.alceml_bdev}')
+            if controller_info:
+                addr = controller_info[0]['ctrlrs'][0]['trid']['traddr']
+                port = controller_info[0]['ctrlrs'][0]['trid']['trsvcid']
+                logger.info(f"IP Address: {addr}:{port}")
+
+            if device.nvmf_multipath:
+                if controller_info and "alternate_trids" in controller_info[0]['ctrlrs'][0]:
+                    addr = controller_info[0]['ctrlrs'][0]['alternate_trids'][0]['traddr']
+                    port = controller_info[0]['ctrlrs'][0]['alternate_trids'][0]['trsvcid']
+                    logger.info(f"IP Address: {addr}:{port}")
+
+                if bdev_info:
+                    logger.info(f"multipath policy: {bdev_info[0]['driver_specific']['mp_policy']}")
 
     return result
 
@@ -456,8 +689,9 @@ def check_lvol_on_node(lvol_id, node_id, node_bdev_names=None, node_lvols_nqns=N
         logger.error(f"lvol not found: {lvol_id}")
         return False
 
-    snode = db_controller.get_storage_node_by_id(node_id)
-    if not snode:
+    try:
+        snode = db_controller.get_storage_node_by_id(node_id)
+    except KeyError:
         return False
 
     rpc_client = RPCClient(
@@ -467,8 +701,9 @@ def check_lvol_on_node(lvol_id, node_id, node_bdev_names=None, node_lvols_nqns=N
     if not node_bdev_names:
         node_bdev_names = {}
         ret = rpc_client.get_bdevs()
-        for bdev in ret:
-            node_bdev_names[bdev['name']] = bdev
+        if ret:
+            for bdev in ret:
+                node_bdev_names[bdev['name']] = bdev
 
     if not node_lvols_nqns:
         node_lvols_nqns = {}
@@ -483,28 +718,9 @@ def check_lvol_on_node(lvol_id, node_id, node_bdev_names=None, node_lvols_nqns=N
             if bdev_info['type'] in ["bdev_lvol", "bdev_lvol_clone"]:
                 bdev_name = lvol.lvol_uuid
 
-            if bdev_name in node_bdev_names:
-                logger.info(f"Checking bdev: {bdev_name} ... ok")
-            else:
-                logger.error(f"Checking bdev: {bdev_name} ... failed")
-                passed = False
+            passed &= check_bdev(bdev_name, bdev_names=node_bdev_names)
 
-        if lvol.nqn in node_lvols_nqns:
-            logger.info(f"Checking subsystem ... ok")
-            if node_lvols_nqns[lvol.nqn]["listen_addresses"]:
-                logger.info(f"Checking listener ... ok")
-            else:
-                logger.info(f"Checking listener ... not found")
-                passed = False
-
-            if node_lvols_nqns[lvol.nqn]["namespaces"]:
-                logger.info(f"Checking namespaces ... ok")
-            else:
-                logger.info(f"Checking namespaces ... not found")
-                passed = False
-        else:
-            logger.info(f"Checking subsystem ... not found")
-            passed = False
+        passed &= check_subsystem(lvol.nqn, nqns=node_lvols_nqns, ns_uuid=lvol.uuid)
 
     except Exception as e:
         logger.exception(e)
@@ -529,7 +745,7 @@ def check_lvol(lvol_id):
         passed = True
         for nodes_id in lvol.nodes:
             node = db_controller.get_storage_node_by_id(nodes_id)
-            if node and node.status == StorageNode.STATUS_ONLINE:
+            if node.status == StorageNode.STATUS_ONLINE:
                 ret = check_lvol_on_node(lvol_id, nodes_id)
                 if not ret:
                     passed = False
@@ -580,14 +796,11 @@ def check_jm_device(device_id):
     try:
         rpc_client = RPCClient(
             snode.mgmt_ip, snode.rpc_port,
-            snode.rpc_username, snode.rpc_password, timeout=3, retry=2)
+            snode.rpc_username, snode.rpc_password, timeout=5, retry=2)
 
-        ret = rpc_client.get_bdevs(jm_device.jm_bdev)
-        if ret:
-            logger.debug(f"Checking bdev: {jm_device.jm_bdev} ... ok")
-        else:
-            logger.debug(f"Checking bdev: {jm_device.jm_bdev} ... not found")
-            passed = False
+        passed &= check_bdev(jm_device.jm_bdev, rpc_client=rpc_client)
+        if snode.enable_ha_jm:
+            passed &= check_subsystem(jm_device.nvmf_nqn, rpc_client=rpc_client)
 
     except Exception as e:
         logger.error(f"Failed to connect to node's SPDK: {e}")
