@@ -11,12 +11,14 @@ class SnapShot(BaseModel):
     cluster_id: str = ""
     created_at: int = 0
     health_check: bool = True
-    lvol: LVol = None
+    lvol: LVol = None # type: ignore[assignment]
     mem_diff: dict = {}
     pool_uuid: str = ""
     ref_count: int = 0
     size: int = 0
+    used_size: int = 0
     snap_bdev: str = ""
     snap_name: str = ""
     snap_ref_id: str = ""
     snap_uuid: str = ""
+    vuid: int = 0
