@@ -31,27 +31,29 @@ def token_required(f):
                 except Exception as e:
                     print(e)
 
+        headers = {"WWW-Authenticate": 'Basic realm="Login Required"'}
         if not cluster_id or not cluster_secret:
             return {
                 "message": "Authentication Token is missing!",
                 "data": None,
                 "error": "Unauthorized"
-            }, 401
+            }, 401, headers
         try:
             db_controller = DBController()
-            cluster = db_controller.get_cluster_by_id(cluster_id)
-            if not cluster:
+            try:
+                cluster = db_controller.get_cluster_by_id(cluster_id)
+            except KeyError:
                 return {
                     "message": "Invalid Cluster ID",
                     "data": None,
                     "error": "Unauthorized"
-                }, 401
+                }, 401, headers
             if cluster.secret != cluster_secret:
                 return {
                     "message": "Invalid Cluster secret",
                     "data": None,
                     "error": "Unauthorized"
-                }, 401
+                }, 401, headers
         except Exception as e:
             return {
                 "message": "Something went wrong",
